/* GameState.qml
 *
 * Copyright 2013-2014 Jamie Strandboge <jamie@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import "../components/util.js" as Util

Item {
    id: state

    property var score: 0
    property var found: []
    property var letters: []
    property var twos: []
    property var playing: false
    property var solved: []

    function reset() {
        score = 0
        found = null
        found = []
        letters = null
        letters = []
        twos = null
        twos = []
        playing = false
        solved = null
        solved = []

        /* DEBUGGING
        var start = new Date()

        // 4x4 initialBoard
        letters = Util.initialBoard("4")
        solved = Util.solveBoard(letters,
                                     4,
                                     storage.dictTrie,
                                     3,
                                     4 * 4)
        // 5x5 initialBoard
        letters = Util.initialBoard("5")
        solved = Util.solveBoard(letters,
                                     5,
                                     storage.dictTrie,
                                     3,
                                     5 * 5)
        // 6x6 initialBoard
        letters = Util.initialBoard("6")
        solved = Util.solveBoard(letters,
                                     6,
                                     storage.dictTrie,
                                     3,
                                     6 * 6)
        var end = new Date()

        Util.debug(" solved in: " + (end.getTime() - start.getTime()) / 1000)
        Util.debug("GameState: solved = ")
        for (var i = 0; i < solved.length; i++) {
           console.log(" " + solved[i])
        }
        //Qt.quit()
        */
    }

    signal gameStarted()
    signal gameStopped()

    onScoreChanged: {
        if (found.length === 1) {
            gameStarted()
        }
        Util.debug("GameState: score = " + score)
    }

    onFoundChanged: {
        if (found === null || found === undefined) {
            Util.debug("GameState: found = null")
        } else {
            Util.debug("GameState: found = ")
            for (var i = 0; i < found.length; i++) {
                Util.debug(" " + found[i])
            }
        }
    }

    onLettersChanged: {
        var s = Util.getBoardStr(letters)
        if (s === "") {
            return
        }
        Util.debug("GameState: letters = ")
        Util.debug("\n" + s)

        if (storage.dictTrie !== null) {
            var start = new Date()
            solved = Util.solveBoard(letters,
                                     storage.getSetting("rules", "gridSize"),
                                     storage.dictTrie,
                                     storage.getSetting("rules", "minimumLength"),
                                     storage.getSetting("rules", "gridSize") *
                                     storage.getSetting("rules", "gridSize"))
            var end = new Date()
            Util.debug("GameState: solved in " +
                       ((end.getTime() - start.getTime()) / 1000) +
                       " seconds (" + solved.length + " words found)")
            Util.debug("GameState: solved = ")
            s = ""
            for (var i = 0; i < solved.length; i++) {
                s = s + " " + solved[i] + "\n"
            }
            Util.debug(s)
        }
    }

    onPlayingChanged: {
        Util.debug("GameState: playing = " + playing)
        if (playing === false) {
            gameStopped()
        }
    }

    function getWordScore(word) {
        return Util.findScore(word, twos, storage.dictAlphabet)
    }
}
