/* GameSummary.qml
 *
 * Copyright 2013-2014 Jamie Strandboge <jamie@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Popups 0.1
import "../components/util.js" as Util

Item {
    id: gameSummaryDialog
    property alias dialog: dialogComponent
    property var found: null

    Component {
        id: dialogComponent

        PopupBase {
            id: dialog
            __foreground: foreground
            __dimBackground: true

            // give us the focus
            __eventGrabber.enabled: true
            property Item pointerTarget: gameSummaryDialog

            anchors.fill: parent

            Rectangle {
                id: foreground
                height: parent.height
                width: parent.width
                color: "transparent"

                Column {
                    anchors.horizontalCenter: parent.horizontalCenter
                    anchors.verticalCenter: parent.verticalCenter
                    anchors.margins: units.gu(2)

                    width: parent.width / 1.5
                    spacing: units.gu(2)

                    Label {
                        anchors.horizontalCenter: parent.horizontalCenter
                        font.bold: false
                        fontSize: "x-large"
                        text: gameState.playing ?
                                  i18n.tr("Game Statistics") :
                                  i18n.tr("Game Over")
                    }

                    Grid {
                        id: summaryGrid
                        columns: 2
                        spacing: units.gu(1)

                        // score
                        Label {
                            font.bold: true
                            fontSize: "large"
                            text: i18n.tr("Score: ")
                        }
                        Label {
                            fontSize: "large"
                            text: gameState.score
                        }

                        // words found
                        Label {
                            font.bold: true
                            fontSize: "large"
                            text: i18n.tr("Total: ")
                        }
                        Label {
                            fontSize: "large"
                            text: found.length + "/" + gameState.solved.length
                        }
                    }

                    Column {
                        id: wordsColumn
                        //spacing: units.gu(1.5)
                        spacing: units.gu(1.8)
                        width: parent.width
                        anchors.horizontalCenter: parent.horizontalCenter
                        anchors.margins: units.gu(2)

                        // TODO: may need to be looking at fontSize in this
                        // calculation...
                        height: Math.min(wordsLength(gameState.playing) *
                                         units.gu(2.2) + units.gu(3.6),
                                         (parent.height * 2 / 3) - units.gu(3.6))

                        Label {
                            font.bold: true
                            fontSize: "large"
                            text: gameState.playing ?
                                    i18n.tr("Words found: ") :
                                    i18n.tr("Words:")
                        }
                        Component {
                            id: wordsDelegate
                            Item {
                                height: units.gu(2.5)
                                // The performance of this is not fantastic,
                                // but ok for now. Probably better to use an
                                // Icon
                                Rectangle {
                                    id: wordShape
                                    height: units.gu(1)
                                    width: height
                                    rotation: 45
                                    color: wordShapeColor(index, gameState.playing)
                                    anchors.verticalCenter: parent.verticalCenter
                                    anchors.horizontalCenter: parent.horizontalCenter
                                    visible: wordShapeVisible(index, gameState.playing)
                                }
                                Label {
                                    anchors.left: wordShape.right
                                    anchors.leftMargin: units.gu(0.8)
                                    fontSize: "large"
                                    text: wordText(index, gameState.playing)
                                }
                            }
                        }
                        ListView {
                            model: wordsLength(gameState.playing)
                            height: wordsColumn.height - units.gu(3.6)
                            width: wordsColumn.width
                            delegate: wordsDelegate
                            visible: true
                        }
                    }
                    Rectangle {
                        height: dismissButton.height + units.gu(2)
                        width: parent.width
                        color: "transparent"
                        Button {
                            id: dismissButton
                            //anchors.centerIn: parent
                            anchors.horizontalCenter: parent.horizontalCenter
                            anchors.bottom: parent.bottom
                            text: i18n.tr("Dismiss")
                            onClicked: {
                                Util.debug("Finished game summary")
                                PopupUtils.close(dialog)
                            }
                        }
                    }
                }
            }
        }
    }

    function wordsLength(playing) {
        return playing === true ? found.length : gameState.solved.length
    }

    function wordShapeColor(index, playing) {
        // only show the shape when solved is shown and then only if the word
        // is in solved but not in found
        var c = storage.getSetting("theme", "correctColor")
        if (playing === false && found.indexOf(gameState.solved[index]) < 0) {
            c = storage.getSetting("theme", "incorrectColor")
        }
        return c
    }

    function wordShapeVisible(index, playing) {
        return playing ? false : true
    }

    function wordText(index, playing) {
        var s = ""
        if (playing === true) {
            s = found[index] + " (" + gameState.getWordScore(found[index]) +
                i18n.tr(" points)")
        } else if (found.indexOf(gameState.solved[index]) >= 0) {
            s = gameState.solved[index] +
                " (" + gameState.getWordScore(gameState.solved[index]) +
                i18n.tr(" points)")
        } else {
            s = gameState.solved[index]
        }
        return s
    }
}
