/* trie.js
 *
 * Based on by https://github.com/jeresig/trie-js
 * Copyright 2011 John Resig
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 *
 * Modifications:
 * Copyright 2013-2014 Jamie Strandboge <jamie@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


/*
 * This takes about .5 seconds to build on the fly. We could possibly optimize
 * this by caching the result. What we have in our favor is it is only once
 * per dictionary load.
 */
function buildTrie(words) {
    var trie = {}
    for (var i = 0, l = words.length; i < l; i++) {
        var word = words[i]
        var letters = word.split("")
        var cur = trie

        for (var j = 0; j < letters.length; j++) {
            var letter = letters[j]

            var pos = null
            if (cur[letter] !== undefined) {
                pos = cur[letter]
            }

            if (pos === null) {
                cur = cur[letter] = (j === letters.length - 1 ? 0 : {})
            } else if (pos === 0) {
                cur = cur[letter] = { $: 0 }
            } else {
                cur = cur[letter]
            }
        }
    }

    return trie
}
