/* util.js
 *
 * Copyright 2013-2014 Jamie Strandboge <jamie@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

var debugging = false

function debug(m) {
    if (debugging === true) {
        print("[DEBUG] " + m)
    }
}

// QML doesn't offer in place array sorting of the form: arr.sort(). As such,
// implement arraySort() to return a sorted array
function arraySort(arr) {
    if (arr === null) {
        return null
    }

    var tmp = null
    tmp = Array()
    for (var i = 0; i < arr.length; i++) {
        tmp.push(arr[i])
    }
    tmp.sort()
    return tmp
}

//
// Letters functions
//

// Initial boards
function initialBoard(gridSize) {
        var initial_letters = null
        switch (gridSize) {
        case "4":
            initial_letters = [
                "b",
                "l",
                "a",
                "b",

                "u",
                "b",
                "u",
                "b",

                "d",
                "u",
                "n",
                "l",

                "j",
                "a",
                "t",
                "e"
            ]
            break
        case "5":
            initial_letters = [
                "b",
                "l",
                "a",
                "b",
                "b",

                "u",
                "b",
                "u",
                "n",
                "l",

                "f",
                "i",
                "n",
                "t",
                "e",

                "a",
                "l",
                "d",
                "u",
                "h",

                "l",
                "s",
                "r",
                "o",
                "w"
            ]
            break
        case "6":
            initial_letters = [
                "b",
                "l",
                "a",
                "b",
                "b",
                "l",

                "u",
                "b",
                "u",
                "n",
                "t",
                "e",

                "f",
                "i",
                "n",
                "d",
                "u",
                "h",

                "a",
                "l",
                "l",
                "s",
                "w",
                "o",

                "n",
                "i",
                "h",
                "i",
                "d",
                "r",

                "e",
                "d",
                "t",
                "b",
                "o",
                "a"
            ]
            break
        }

        return initial_letters
}

// Get string representation of the board
function getBoardStr(letters) {
    var s = ""
    if (letters === null || letters === undefined || letters.length === 0) {
        return s
    }

    var column = 0
    for (var i = 0; i < letters.length; i++) {
        if (i !== 0 && column % storage.getSetting("rules", "gridSize") === 0) {
            s += "\n"
        }
        s += " " + letters[i]
        column++
    }
    return s
}

// is tile adjacent
function isAdjacent(cur, next, gridSize) {
    var col_cur = cur % gridSize
    var col_next = next % gridSize
    var row_cur = Math.floor(cur / gridSize)
    var row_next = Math.floor(next / gridSize)

    if ((col_next === col_cur && row_next === row_cur) ||
        col_next < col_cur - 1 ||
        col_next > col_cur + 1 ||
        row_next < row_cur - 1 ||
        row_next > row_cur + 1) {
        return false
    }
    return true
}

// return an array of letters based on the alphabet
function newBoard(alphabet, gridSize) {
    var letters = []
    var dist = []

    // setup distribution array
    for (var letter in alphabet) {
        for (var i = 0; i < alphabet[letter]["num"]; i++) {
            dist.push(letter)
        }
    }

    // pick random letters from the distribution array, removing tiles as
    // we use them
    for (var i = 0; i < gridSize * gridSize; i++) {
        var rnd = Math.floor(Math.random() * (dist.length - 0))
        letters.push(dist[rnd])
        dist.splice(rnd, 1)
    }
    return letters
}

// solve
function getIndex(column, row, gridSize) {
    if (column < 0 || column >= gridSize || row < 0 || row >= gridSize) {
        return -1
    }
    return column + gridSize * row
}

function _inTrie(word, cur) {
    var letters = word.split("")

    /*
    var s = "_inTrie('" + word + "', {"
    for (var k in cur) {
        s = s + k + ","
    }
    s = s + "})"
    console.log(s)
    */

    for (var i = 0, l = letters.length; i < l; i++) {
        var c = letters[i]
        if (cur !== 0 && c in cur) {
            cur = cur[c]
            /*
            s = " cur[" + c + "]: {"
            for (var k in cur) {
                s = s + k + ","
            }
            s = s + "})"
            console.log(s)
            */
        } else {
            //console.log(" not a word (not in trie): " + word)
            return -1
        }
    }

    if (cur === 0 || "$" in cur) { // matches here
        //console.log(" is word: " + word)
        return 0
    }

    //console.log(" not a word (in trie): " + word)
    return 1
}

function crawl(puz, column, row, word, trie, used) {
    if (word.length >= puz.min) {
        var rc = _inTrie(word, trie)
        if (rc < 0) {
            return
        }
        if (rc === 0 && puz.found.indexOf(word) < 0) {
            //console.log(" found word: " + word)
            puz.found.push(word)
        }
    }

    var dcol = [-1, 0, 1]
    var drow = [-1, 0, 1]
    for (var i = 0; i < dcol.length; i++) {
        for (var j = 0; j < drow.length; j++) {
            // skip if same tile that we started with
            if (dcol[i] === 0 && drow[j] === 0) {
                continue
            }

            var newCol = column + dcol[i]
            var newRow = row + drow[j]
            var idx = getIndex(newCol, newRow, puz.gridSize)

            // skip if out of range or tile already seen
            if (idx < 0 || used.indexOf(idx) >= 0) {
                continue
            }

            var tmp = []
            for (var k = 0; k < used.length; k++) {
                tmp.push(used[k])
            }
            tmp.push(idx)

            // FIXME: optimize to use node instead of top trie
            //console.log("newCol=" + newCol + ",newRow=" + newRow + ",word=" + word + puz.letters[idx])
            crawl(puz, newCol, newRow, word + puz.letters[idx], trie, tmp)
        }
    }
}


function solveBoard(letters, gridSize, trie, minlen, maxlen) {
    if (letters === null || letters === undefined || letters.length === 0) {
        return []
    }
    var puz = new Object()
    puz.letters = letters
    puz.gridSize = gridSize
    puz.min = minlen
    puz.max = maxlen
    puz.found = []

    for (var i = 0; i < gridSize; i++) {
        for (var j = 0; j < gridSize; j++) {
            var idx = getIndex(i, j, puz.gridSize)
            var used = []
            used.push(idx)
            crawl(puz, i, j, puz.letters[idx], trie, used)
        }
    }

    puz.found.sort()
    return puz.found
}


//
// Score functions
//
function findScore(word, twos, alphabet) {
    var chars = word.split('')
    var tmpScore = 0
    for (var i = 0; i < chars.length; i++) {
        var search = chars[i].toLowerCase()
        // check for two-letter matches
        if (i + 1 < word.length &&
           twos.indexOf(chars[i].toLowerCase() +
           chars[i+1].toLowerCase()) >= 0) {
            search = chars[i].toLowerCase() +
                     chars[i+1].toLowerCase()
            i++
        }
        tmpScore += alphabet[search]["pts"]
    }
    return tmpScore
}
