/* AboutTab.qml
 *
 * Copyright 2013-2014 Jamie Strandboge <jamie@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.0
import Ubuntu.Components 0.1

Tab {
    title: i18n.tr("About")

    page: Page {
        anchors.fill: parent
        width: parent.width
        anchors.margins: units.gu(2)
        property var version: "0.3.1"

        Flickable {
            id: myFlickable
            anchors.fill: parent
            contentHeight: aboutColumn.height

            Column {
                id: aboutColumn
                spacing: units.gu(3)
                width: page.width
                y: units.gu(6)

                // Utilize image with transparency
                UbuntuShape {
                    width: 256
                    height: width
                    anchors.horizontalCenter: parent.horizontalCenter
                    UbuntuShape {
                        image: Image {
                            source: "../graphics/blabble256.png"
                        }
                        anchors.fill: parent
                    }
                }
                Grid {
                    anchors.horizontalCenter: parent.horizontalCenter
                    columns: 2
                    spacing: units.gu(1)
                    Label {
                        id: "versionLabel"
                        objectName: "versionLabel"
                        font.bold: true
                        text: i18n.tr("Version: ")
                    }
                    Label {
                        objectName: "version"
                        text: page.version
                    }
                    Label {
                        objectName: "authorLabel"
                        font.bold: true
                        text: i18n.tr("Author: ")
                    }
                    Label {
                        objectName: "author"
                        text: "Jamie Strandboge"
                    }
                    Label {
                        objectName: "licenseLabel"
                        font.bold: true
                        text: i18n.tr("Licenses: ")
                    }
                    Label {
                        objectName: "license"
                        text: "<a href=\"http://www.gnu.org/licenses/gpl-3.0.html#content\">GPL-3</a>"
                        onLinkActivated: Qt.openUrlExternally(link)
                    }
                    Label { text: " "}
                    Label {
                        text: "<a href=\"http://opensource.org/licenses/MIT\">MIT</a> (jsonpath.js, build-trie.js)"
                        onLinkActivated: Qt.openUrlExternally(link)
                    }
                }
                Row {
                    anchors.horizontalCenter: parent.horizontalCenter
                    Label {
                        objectName: "urlLabel"
                        font.bold: true
                        text: "<a href=\"https://launchpad.net/blabble\">https://launchpad.net/blabble</a>"
                        onLinkActivated: Qt.openUrlExternally(link)
                    }
                }

                Row {
                    Text {
                        // TODO: move to template
                        color: versionLabel.color
                        font.pointSize: versionLabel.font.pointSize
                        font.family: versionLabel.font.family
                        wrapMode: Text.WordWrap

                        // FIXME: get wordwrap working
                        text: i18n.tr("<b>Rules:</b><br>
Find as many words as you can by connecting<br>
adjacent tiles. Tiles may only be used once.<br>
Games optionally may be timed and words must<br>
be of minimum length (three by default). Don't<br>
forget the most important rule:<br>
<br>
Have fun! :)
")
                    }
                }

                Row {
                    Text {
                        // TODO: move to template
                        color: versionLabel.color
                        font.pointSize: versionLabel.font.pointSize
                        font.family: versionLabel.font.family

                        text: i18n.tr("<b>ChangeLog:</b><br>
<br>= 0.3.1 =<br>
* Performance improvements during game play<br>
<br>= 0.3 =<br>
* Save settings in U1db<br>
* Also show rectangle for found words<br>
* Highlight already seen word differently<br>
* Process word on out of bounds<br>
* Add Blabble dictionary (ENABLE + extras)<br>
* Add timed games<br>
* Allow setting minimum word length<br>
<br>= 0.2.2 =<br>
* On quit, show unsolved words in stats<br>
<br>= 0.2.1 =<br>
* Solve puzzle and add stat for total words<br>
<br>= 0.2 =<br>
* Initial release<br>
")
                    }
                }

                Row {
                    Text {
                        id: "todoText"

                        // TODO: move to template
                        color: versionLabel.color
                        font.pointSize: versionLabel.font.pointSize
                        font.family: versionLabel.font.family

                        text: i18n.tr("<b>Todo</b>:<br>
* optionally show remaining words by length, etc<br>
* save high scores<br>
* more dictionaries<br>
* implement secondary dictionary<br>
* rotate board<br>
<br>
<br>
")
                    }
                } // Row
            } // Column
        } // Flickable
    } // Page
}
