/* GameBoardGrid.qml
 *
 * Copyright 2013-2014 Jamie Strandboge <jamie@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import "../components"
import "../components/util.js" as Util

Item {
    property alias letters: gameColumn.letters
    property alias score: board.score

    // performance optimizations
    property var gridSize: storage.getSetting("rules", "gridSize")
    property var tileColor: storage.getSetting("theme", "tileColor")
    property var tileGradientColor: storage.getSetting("theme",
                                                       "tileGradientColor")
    property var textColor: storage.getSetting("theme", "textColor")
    property var hiTileColor: storage.getSetting("theme", "hiTileColor")
    property var hiTileGradientColor: storage.getSetting("theme",
                                                     "hiTileGradientColor")
    property var hiTextColor: storage.getSetting("theme", "hiTextColor")
    property var correctColor: storage.getSetting("theme", "correctColor")
    property var correctGradientColor: storage.getSetting("theme",
                                                     "correctGradientColor")
    property var correctTextColor: storage.getSetting("theme",
                                                      "correctTextColor")
    property var incorrectColor: storage.getSetting("theme",
                                                    "incorrectColor")
    property var incorrectGradientColor: storage.getSetting("theme",
                                                     "incorrectGradientColor")
    property var incorrectTextColor: storage.getSetting("theme",
                                                        "incorrectTextColor")
    property var seenColor: storage.getSetting("theme", "seenColor")
    property var seenGradientColor: storage.getSetting("theme",
                                                       "seenGradientColor")
    property var seenTextColor: storage.getSetting("theme", "seenTextColor")

    Timer {
        id: timer
        interval: 1000
        running: false
        repeat: true
        property var seconds: -1
        onTriggered: {
            seconds--
            if (seconds < 0) {
                running = false
                gameTab.quitGame()
                return
            }
            var str = i18n.tr("Time: ")
            var min = Math.floor(seconds / 60)
            var sec = seconds % 60
            timeLabel.text = str + min + ":" + ((sec < 10) ? "0" : "") + sec
        }
    }

    Column {
        id: gameColumn

        spacing: units.gu(2)
        width: parent.width - units.gu(5)
        anchors.horizontalCenter: parent.horizontalCenter

        height: parent.height - units.gu(2)
        anchors.verticalCenter: parent.verticalCenter


        // FIXME
        Label { text: " " }

        Grid {
            id: headerGrid
            width: parent.width
            columns: 2
            Label {
                width: parent.width / 2
                id: scoreLabel
                objectName: "gameTab_scoreLabel"
                fontSize: "large"
                text: i18n.tr("Score: 0")
            }
            Label {
                width: parent.width / 2 - units.gu(1)
                id: timeLabel
                horizontalAlignment: Text.AlignRight
                fontSize: "large"
                text: ""
            }
        }

        // Unplayable board
        property var letters: null
        property var twos: []

        onLettersChanged: {
            if (gameColumn.letters === null ||
                gameColumn.letters === undefined ||
                gameColumn.letters.length === 0) {
                return
            }

            board.score = 0
            board.word = ""
            board.seen = null // for garbage collector
            board.seen = []
            board.found = null // for garbage collector
            board.found = []
            twos = null // for garbage collector
            twos = []
            debugLabel.text = "DEBUG:"

            var tmp = boardGrid.childAt(getX(letters.length - 1), getY(letters.length - 1))
            if (tmp === null) {
                return
            }

            for (var i = 0; i < letters.length; i++) {
                var item = boardGrid.childAt(getX(i), getY(i))
                var points = storage.dictAlphabet[letters[i].toLowerCase()]["pts"]
                item.tileItem.text = _formatTile(i, points)
            }
            // predetermine the two-letter tiles for easy finding later
            for (var i = 0; i < letters.length; i++) {
                if (letters[i].length === 2 && twos.indexOf(letters[i]) < 0) {
                    twos.push(letters[i])
                }
            }
            gameState.letters = letters
            gameState.twos = twos
            updateColor()
        }

        Rectangle {
            // gridSize * 3 is for spacing
            width: parent.width - gridSize * 3
            height: width
            color: "transparent"
            id: board

            property var cellWidth: parent.width / gridSize
            property var cellHeight: cellWidth
            property var word: ""
            property var seen: null
            property var score: 0
            property var found: []
            property var correct: false
            property var already_seen: false

            onScoreChanged: {
                var tmp = i18n.tr("Score: ") + score
                if (score !== 0 && found.length > 0) {
                    tmp += " (" + found.length + " "
                    if (found.length === 1) {
                       tmp += i18n.tr("word")
                    } else {
                       tmp += i18n.tr("words")
                    }
                    tmp += ")"
                }
                scoreLabel.text = tmp
                gameState.score = score
            }

            onFoundChanged: {
                // Note: this is not triggered when the contents of found are
                // updated
                gameState.found = found
            }

            MouseArea {
                anchors.fill: parent
                preventStealing: false

                function getIndex(x, y, fuzzy) {
                    // returns:
                    //   index if in bounds
                    //   -1 if in bounds but no fuzzy match
                    //   -2 if out of bounds
                    if (x < 0 || x > width || y < 0 || y > height) {
                        return -2
                    }

                    var column = Math.floor(x / board.cellWidth)
                    var row = Math.floor(y / board.cellHeight)

                    // return index only if near center
                    var fuzz = Math.round(board.cellWidth / 5)
                    if (fuzzy === true && (
                        x < column * board.cellWidth + fuzz ||
                        x > board.cellWidth - fuzz + column * board.cellWidth ||
                        y < row * board.cellHeight + fuzz ||
                        y > board.cellHeight - fuzz + row * board.cellHeight)) {
                        return -1
                    }

                    return column + gridSize * row
                }

                function isAdjacent(cur, next) {
                    return Util.isAdjacent(cur, next, gridSize)
                }

                // should this be index?
                function highlightTile(x, y) {
                    var item = boardGrid.childAt(x, y)
                    item.color = hiTileColor
                    item.gradientColor = hiTileGradientColor
                    item.tileItem.color = hiTextColor
                }

                function unhighlightTile(x, y) {
                    var item = boardGrid.childAt(x, y)
                    item.color = tileColor
                    item.gradientColor = tileGradientColor
                    item.tileItem.color = textColor
                }

                function animateTiles() {
                    for (var i = 0; board.seen !== null &&
                                    i < board.seen.length; i++) {
                        var x = getX(board.seen[i])
                        var y = getY(board.seen[i])
                        var item = boardGrid.childAt(x, y)
                        item.tileAnimate.running = true
                    }
                }

                function processWord() {
                    if (gameState.playing !== true || board.word === "") {
                        return
                    }
                    Util.debug("processWord()")

                    if (board.word.length <
                       storage.getSetting("rules", "minimumLength")) {
                        debugLabel.text = "DEBUG: '" + board.word + "' too short"
                    } else if (board.word.length > gridSize * gridSize) {
                        // should be impossible
                        debugLabel.text = "DEBUG: '" + board.word + "' too long"
                    } else if (storage.dictWords.indexOf(board.word.toLowerCase()) < 0) { // wrong
                        debugLabel.text = "DEBUG: '" + board.word + "' not found"
                    } else if (board.found.indexOf(board.word.toLowerCase()) >= 0) {
                        debugLabel.text = "DEBUG: '" + board.word +
                                          "' already used"
                        board.already_seen = true
                    } else {
                        debugLabel.text = "DEBUG: '" + board.word + "' found"
                        board.found.push(board.word)
                        board.correct = true
                        board.score += gameState.getWordScore(board.word)
                    }
                    animateTiles()

                    board.seen = null
                    board.word = ""
                    updateColor()
                }

                onReleased: {
                    Util.debug("onReleased")
                    processWord()
                }

                onPressed: {
                    if (gameState.playing !== true) {
                        return
                    }
                    board.correct = false
                    board.already_seen = false
                    board.seen = []
                    var idx = getIndex(mouseX, mouseY, false)
                    board.seen.push(idx)
                    board.word = gameColumn.letters[idx]
                    highlightTile(mouseX, mouseY)
                }

                onPositionChanged: {
                    if (gameState.playing !== true) {
                        return
                    }
                    if (board.seen === null) {
                        return
                    }
                    var idx = getIndex(mouseX, mouseY, true)
                    if (idx < 0) {
                        // TODO: perhaps going off the screen should add
                        // the word
                        if (idx === -2) {
                            Util.debug("onPositionChanged: out of bounds")
                            processWord()
                        }
                    } else {
                        if (idx !== board.seen[board.seen.length-1] &&
                            board.seen.indexOf(idx) < 0 &&
                            Util.isAdjacent(board.seen[board.seen.length-1],
                                            idx, gridSize)) {
                            board.seen.push(idx)
                            board.word += gameColumn.letters[idx]
                            highlightTile(mouseX, mouseY)
                        } else if (idx === board.seen[board.seen.length-2]) {
                            // unselect
                            unhighlightTile(getX(board.seen[board.seen.length-1]),
                                            getY(board.seen[board.seen.length-1]))
                            board.word = board.word.substring(0,
                                         board.word.length - 1)
                            board.seen.pop()
                        }
                    }
                }
            }

            Grid {
                columns: gridSize
                anchors.horizontalCenter: parent.horizontalCenter
                id: boardGrid
                width: parent.width - gridSize

                // TODO: needing this at all is janky. clean this up
                function _isReady(index) {
                    if (gameColumn.letters === undefined ||
                        gameColumn.letters === null ||
                        gameColumn.letters[index] === undefined ||
                        storage.dictAlphabet === null ||
                        storage.dictAlphabet[gameColumn.letters[index].toLowerCase()] === undefined ||
                        storage.dictAlphabet[gameColumn.letters[index].toLowerCase()]["pts"] === null) {
                         return false
                     }
                     return true
                }

                spacing: 3
                Repeater {
                    id: repeater
                    model: gridSize * gridSize

                    UbuntuShape {
                        id: "tileShape"
                        width: parent.width / gridSize
                        height: width
                        radius: "small"
                        color: tileColor
                        gradientColor: tileGradientColor
                        property var points: (boardGrid._isReady(index) === true) ? storage.dictAlphabet[gameColumn.letters[index].toLowerCase()]["pts"] : "0"

                        property alias tileItem: tileText

                        Label {
                            id: "tileText"
                            font.capitalization: Font.Capitalize
                            anchors.centerIn: parent
                            fontSize: "x-large"
                            color: textColor
                            text: _formatTile(index, points)
                            textFormat: Text.RichText
                        }

                        property alias tileAnimate: animationTimer
                        Timer {
                            id: "animationTimer"
                            interval: 150
                            //interval: 1000 // for testing animation
                            running: false
                            repeat: false
                            triggeredOnStart: true
                            onTriggered: {
                                var animColor = incorrectColor
                                var animGradientColor = incorrectGradientColor
                                var animTextColor = incorrectTextColor
                                if (board.correct === true) {
                                    animColor = correctColor
                                    animGradientColor = correctGradientColor
                                    animTextColor = correctTextColor
                                } else if (board.already_seen === true) {
                                    animColor = seenColor
                                    animGradientColor = seenGradientColor
                                    animTextColor = seenTextColor
                                }
                                if (running === true) {
                                    tileShape.color = animColor
                                    tileShape.gradientColor = animGradientColor
                                    tileItem.color = animTextColor
                                } else {
                                    tileShape.color = tileColor
                                    tileShape.gradientColor = tileGradientColor
                                    tileItem.color = textColor
                                }
                            }
                        }
                    }
                }
            }
        }

        Label {
            id: debugLabel
            objectName: "gameTab_debugLabel"
            fontSize: "large"
            text: i18n.tr("DEBUG: ")
            // change to false when in production
            visible: false
        }
    }

    function getX(idx) {
        if (idx < 0 || idx >= gridSize * gridSize) {
            return -1
        }
        return (idx % gridSize * board.cellWidth + board.cellWidth / 2)
    }

    function getY(idx) {
        if (idx < 0 || idx >= gridSize * gridSize) {
            return -1
        }
        return Math.floor(idx / gridSize) * board.cellHeight +
                   board.cellHeight / 2
    }

    function updateColor() {
        for (var i = 0; i < boardGrid.children.length; i++) {
            var item = boardGrid.children[i]
            if (item !== undefined && item.hasOwnProperty("color")) {
                boardGrid.children[i].color = tileColor
                boardGrid.children[i].gradientColor = tileGradientColor
                boardGrid.children[i].tileItem.color = textColor
            }
        }
    }

    function newGame() {
        // setup the board
        letters = null
        letters = Util.newBoard(storage.dictAlphabet, gridSize)
        updateColor()
        if (storage.getSetting("rules", "time") !== "Unlimited") {
            timer.seconds = storage.getSetting("rules", "time").split(" ")[0] * 60
            timeLabel.text = i18n.tr("Time: ") + storage.getSetting("rules", "time").split(" ")[0] + ":00"
            Util.debug("Started timer")
            timer.running = true
        } else {
            timeLabel.text = ""
            timer.running = false
            timeLabel.text = ""
            Util.debug("Stopped timer")
        }
        Util.debug("Game started with new letters")
    }

    function initializeBoard() {
        gridSize = storage.getSetting("rules", "gridSize")
        letters = null
        letters = Util.initialBoard(gridSize)
        Util.debug("Initialized letters")
        if (storage.getSetting("rules", "time") !== "Unlimited") {
            timer.seconds = storage.getSetting("rules", "time").split(" ")[0] * 60
            timer.running = false
            timeLabel.text = i18n.tr("Time: ") + storage.getSetting("rules", "time").split(" ")[0] + ":00"
            Util.debug("Initialized timer")
        } else {
            timeLabel.text = ""
        }
    }

    function _formatTile(index, points) {
        if (gameColumn.letters === null ||
            gameColumn.letters === undefined ||
            gameColumn.letters[index] === null) {
            return ""
        }
        var letter = gameColumn.letters[index]

        var size = 1
        switch(gridSize) {
        case "4":
            size = 1.8
            break
        case "5":
            size = 1.5
            break
        case "6":
            size = 1.3
            break
        }
        var s = ""
        if (letter !== undefined && letter.length > 1) {
            s = "<span style=\"font-size:" + units.gu(size * 2.5) + "px\">" +
                letter + "</span><span style=\"font-size:" + units.gu(size) +
                "px\">" + points + "</span>"

        } else {
            s = "<span style=\"font-size:" + units.gu(size * 2.5) +
                    "px\">&nbsp;" + letter + "</span><span style=\"font-size:" +
                    units.gu(size) + "px\">&nbsp;" + points + "</span>"
        }

        return s
    }
}
