/* GameTab.qml
 *
 * Copyright 2013-2014 Jamie Strandboge <jamie@ubuntu.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.Popups 0.1
import "../components"
import "../components/util.js" as Util

Tab {
    title: i18n.tr("Blabble")
    id: gameTab

    // mainView uses these
    property alias gameState: myState
    property alias gameSummary: mySummary

    GameSummary {
        id: mySummary
    }

    GameState {
        id: myState
        onGameStarted: {
            quitGameButton.visible = playing
            statsButton.visible = playing
        }
        onGameStopped: quitGameButton.visible = playing
    }

    page: Page {
        tools: ToolbarItems {
            ToolbarButton {
                id: newGameButton
                objectName: "newGameButton"
                action: Action {
                    text: i18n.tr("New")
                    iconSource: "image://theme/add"
                    onTriggered: {
                        page.tools.opened = false
                        startGame()
                    }
                }
            }
            ToolbarButton {
                id: statsButton
                objectName: "statsButton"
                visible: false

                action: Action {
                    text: i18n.tr("Stats")
                    iconSource: "image://theme/search"
                    onTriggered: {
                        page.tools.opened = false
                        showStats()
                    }
                }
            }
            ToolbarButton {
                id: quitGameButton
                objectName: "quitGameButton"
                visible: false

                action: Action {
                    text: i18n.tr("Quit")
                    iconSource: "image://theme/close"
                    onTriggered: {
                        page.tools.opened = false
                        quitGame()
                    }
                }
            }
        }

        // Containing rectangle for dynamically created GameBoard object
        Rectangle {
            id: myGameBoard
            anchors.fill: parent
            color: "transparent"

            // access the GameBoard through this object
            property var obj: null
        }
    }

    function createGameBoard() {
        // reset the gameState
        gameState.reset()
        statsButton.visible = false

        // dynamically create the GameBoard object
        if (myGameBoard.obj !== null) {
            myGameBoard.obj.destroy()
        }

        var c = Qt.createComponent("GameBoard.qml")
        var obj = c.createObject(myGameBoard, {});
        if (obj === null) {
            console.log("Error creating object")
            return
        }

        // now, attach it to our containing rectangle
        myGameBoard.obj = obj

        // initialize the board
        myGameBoard.obj.initializeBoard()
    }

    function startGame() {
        gameState.reset()
        gameState.playing = true
        statsButton.visible = false
        myGameBoard.obj.newGame()
    }

    function showStats() {
        // Only sort on demand
        gameSummary.found = Util.arraySort(gameState.found)
        PopupUtils.open(gameSummary.dialog)
    }

    function quitGame() {
        updateColor() // clear tile highlights if game stopped during play
        gameState.playing = false
        showStats()
        Util.debug("Game stopped")
    }

    function updateColor() {
        // used when changing the theme
        if (myGameBoard.obj !== null) {
            myGameBoard.obj.updateColor()
        }
    }
}
